/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.utils.xml.xpath;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import javax.xml.xpath.XPathFunction;
import javax.xml.xpath.XPathFunctionException;

import org.w3c.dom.NodeList;


/**
 * Execute the DateAfter() function.<br/>
 * Usage: <tt>xem:DateAfter(date1,"date2")</tt><br/>
 * Returns <tt>TRUE</tt> if and only if <tt>date1</tt> is <strong>after</strong> and not equal <tt>date2</tt></br>
 * Dates are supposed to respect <tt>yyyy-MM-dd</tt> pattern
 */
public class FuncDateAfter extends AbstractXPathFunction {
	private static org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(FuncDateAfter.class);
	public static final transient String FUNCTION_NAME = "DateAfter";
    

    public Object evaluate(List args) throws XPathFunctionException {
        if(args.size()!=2) throw new XPathFunctionException("DateAfter requires exactly 2 parameters");
        Object o1 = args.get(0);
        Object o2 = args.get(1);
        String s1=null, s2=null;
        s1 = getStringValue(o1);
        s2 = getStringValue(o2);
        if(s1==null && s2==null) return Boolean.FALSE;
        if(s1==null || s2==null) return Boolean.FALSE;
        Boolean ret = Boolean.FALSE;
        try {
            DatatypeFactory df = DatatypeFactory.newInstance();
            XMLGregorianCalendar d1 = df.newXMLGregorianCalendar(s1);
            XMLGregorianCalendar d2 = df.newXMLGregorianCalendar(s2);
            ret = new Boolean(d1.compare(d2) > 0);
        } catch(DatatypeConfigurationException dcEx) {
            logger.error("",dcEx);
        }
        return ret;
    }


}
