/**
 * Definit le contrat de service pour les importeurs.
 * Ce sont les differentes implementations de cette interface
 * qui interagiront avec l'utilisateur via une interface swing,
 * une ligne de commande, un webservice, etc...
 */

package fr.gouv.finances.cp.xemelios.importers;

import fr.gouv.finances.cp.utils.Pair;
import fr.gouv.finances.cp.xemelios.common.ToolException;
import fr.gouv.finances.cp.xemelios.common.config.DocumentModel;

public interface ImportServiceProvider {
	public final int MSG_TYPE_INFORMATION = 1;
	public final int MSG_TYPE_ERROR = 2;
	
	public Pair getBudget(DocumentModel dm, Pair collectivite, String fileName) throws ToolException;
	public Pair getCollectivite(DocumentModel dm, String fileName) throws ToolException;
	public boolean getOverwrite(String docName) throws Exception;
	public void pushCurrentProgress(int progress) throws NullPointerException;
	public void displayProgress(final int nbFiles);
	public void displayMessage(String msg, int msgType);
	public void displayException(Throwable t);
//	public void budgetInteractif(boolean value);
//	public void collectiviteInteractif(boolean value);
	public void startLongWait();
	public void endLongWait();
    public void setDisplayFeedback(boolean display);
    public boolean shouldDisplayFeedback();
    public void setCollectivite(Pair p);
    public void setBudget(Pair p);
}
