/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2007 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.updater.script.config;

import java.util.ArrayList;
import java.util.Hashtable;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;
import javax.xml.namespace.QName;

public class XemeliosComponentModel implements XmlMarshallable {
	public static final transient String TAG = "xemelios-components";
        public static final transient QName QN =new QName(TAG);
	private ArrayList<ComponentModel> components;
	private Hashtable<String,ComponentModel> compsById;

	public XemeliosComponentModel(QName tagName) {
		super();
		components = new ArrayList<ComponentModel>();
		compsById = new Hashtable<String,ComponentModel>();
	}

	public void addCharacterData(String cData) throws SAXException { }

	public void addChild(XmlMarshallable child, QName tagName) throws SAXException {
		ComponentModel cm = (ComponentModel)child;
		components.add(cm);
		compsById.put(cm.getId(),cm);
	}

	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
		return this;
	}

	public void marshall(XmlOutputter output) {
		output.startTag(TAG);
		for(ComponentModel cm:components) cm.marshall(output);
		output.endTag(TAG);
	}

	public void validate() throws InvalidXmlDefinition {
	}
	
	@Override
	public XemeliosComponentModel clone() {
		return this;
	}

	public ArrayList<ComponentModel> getComponents() {
		return components;
	}
	
	public String getComponentVersion(String id) {
		ComponentModel cm = compsById.get(id);
		if(cm==null) return null;
		return cm.getVersion();
	}
	public void remove(String id) {
		ComponentModel cm = compsById.get(id);
		if(cm!=null) components.remove(cm);
		compsById.remove(id);
	}

    public XmlMarshallable getChildToModify(String uri, String localName, String qName, Attributes atts) {
        return null;
    }

    public QName getQName() {
        return QN;
    }
}
